//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// ImageButton class - graphical pushbutton
//
import java.awt.Graphics;
import java.awt.Frame;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Image;
import java.awt.image.ImageProducer;
import java.awt.AWTEventMulticaster;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ResourceBundle;

/**
 * Displays a graphical button.
 */
public class ImageButton extends ImageLabel
			implements MouseListener {

	// colour catalog information
	private static final ResourceBundle	colours		=
				ResourceBundle.getBundle( "ColorResource" );

	// button dimensions, generally larger than the superclass's
	private Dimension		pref_size;//	= null;
	private static final int	SIZE_DELTA	= 4;

	// label to display under the image - null if no label
	private String			label;//	= null;

	// padding above and below the label, and label fontmetrics
	private static final int	PADDING		= 4;
	private FontMetrics		fm;//		= null;

	// colour for text when the mouse is in the button, and for when the
	// button is disabled
	private Color			mouseover_fg;//	= null;
	private Color			disabled_fg;//	= null;

	// flags indicating where the mouse is in this button
	private boolean			mouse_pressed;//	= false;
	private boolean			mouse_entered;//	= false;

	// action listener multicast list head for button actions
	private ActionListener		action_listener;//	= null;


	/*
	 * Internal initialization.
	 */
	private void internal_initialize() {

		// find the longest side of a graphic that consists of the
		// image, and the label, if any, beneath it
		Dimension graphic_dimension = super.getPreferredSize();
		int dimension;
		if ( label == null )
			dimension = Math.max( graphic_dimension.width,
						graphic_dimension.height);
		else {
			// note the normal and mouseover foregrounds
			setForeground( Colour.find_colour(
				colours.getString( "ImageButton.foreground" )));
			mouseover_fg = Colour.find_colour(
				colours.getString( "ImageButton.mouseoverfg" ));
			disabled_fg = Colour.find_colour(
				colours.getString( "ImageButton.disabledfg" ));

			// set the font for xpmbuttons, and get the metric
			setFont( Font.decode(
				colours.getString( "ImageButton.font" )));
			fm = getFontMetrics( getFont() );

			dimension = Math.max( graphic_dimension.width,
					Math.max(
					fm.stringWidth( label ) + PADDING,
					graphic_dimension.height
					+ fm.getAscent() + PADDING));
		}

		// set this as our preferred size
		pref_size = new Dimension( dimension + SIZE_DELTA,
						dimension + SIZE_DELTA );
	}

	/**
	 * Create an ImageButton from an image, with label text.
	 */
	public ImageButton( Image image, String text ) {
		super( image );

		// save the label passed in to the constructor
		this.label = text;

		// initialize the dimensions, and watch for mouse events
		internal_initialize();
		addMouseListener( this );
	}

	/**
	 * Create an ImageButton from an image, without label text.
	 */
	public ImageButton( Image image ) {
		this( image, null );
	}

	/**
	 * Create a new ImageButton initialized with the given ImageProducer,
	 * and with label text.
	 */
	public ImageButton( ImageProducer imageproducer, String text ) {
		super( imageproducer );

		// save the label passed in to the constructor
		this.label = text;

		// initialize the dimensions, and watch for mouse events
		internal_initialize();
		addMouseListener( this );
	}

	/**
	 * Create a new ImageButton initialized with the given ImageProducer,
	 * and without label text.
	 */
	public ImageButton( ImageProducer imageproducer ) {
		this( imageproducer, null );
	}

	/*
	 * Return dimensions.
	 */
	public Dimension getPreferredSize() { return ( pref_size ); }
	public Dimension getMinimumSize() { return ( pref_size ); }

	/**
	 * Update the graphic.
	 */
	public void update( Graphics g ) {
		Dimension d = getSize();

		// if the mouse is pressed, darken the background colour
		g.setColor( mouse_pressed ? getBackground().darker()
						: getBackground() );
		g.fillRect( 0, 0, d.width - 1, d.height - 1 );

		// see if we have a label to add to the graphic
		if ( label != null ) {

			// render the image, leaving space for the label
			Graphics gcopy = g.create();
			gcopy.translate( 0, -fm.getAscent() + PADDING );
			super.update( gcopy );

			// set the label colour
			if ( ! isEnabled() )
				g.setColor( disabled_fg );
			else
				g.setColor( mouse_entered ?
					mouseover_fg : getForeground() );

			// add the label under the image
			g.drawString( label,
				( d.width - fm.stringWidth( label )) / 2,
				d.height - PADDING / 2 );
		}
		else
			// simply update the image
			super.update( g );

		// add a box if mouse entered, pushed in if button pressed,
		// and we are enabled
		g.setColor( getBackground() );
		if ( isEnabled() && mouse_pressed )
			// draw a sunken box
			g.draw3DRect( 0, 0, d.width - 1, d.height - 1, false );
		else if ( isEnabled() && mouse_entered )
			// draw a raised box
			g.draw3DRect( 0, 0, d.width - 1, d.height - 1, true );
		else {
			// erase all traces of a box
			g.drawRect( 0, 0, d.width - 1, d.height - 1 );
			g.drawRect( 1, 1, d.width - 2, d.height - 2 );
		}
	}

	/**
	 * Paint the graphic.
	 */
	public void paint( Graphics g ) { update( g ); }

	/**
	 * Add an action listener.
	 */
	public void addActionListener( ActionListener new_action_listener ) {
		action_listener = AWTEventMulticaster.add(
					action_listener, new_action_listener );
	}

	/**
	 * Remove an action listener.
	 */
	public void removeActionListener( ActionListener old_action_listener ) {
		action_listener = AWTEventMulticaster.remove(
					action_listener, old_action_listener );
	}

	/**
	 * Intercept calls to enable and disable the button.
	 */
	public void setEnabled( boolean enabled ) {
		super.setEnabled( enabled );

		// if disabling, reset mouse tracking flags
		if ( ! enabled ) {
			mouse_entered = false; mouse_pressed = false;
		}
		repaint();
	}

	/**
	 * Event handlers.
	 */
	public void mouseClicked( MouseEvent e ) {

		// distribute an action event to all listeners, using the
		// event-safe multicaster
		if ( action_listener != null ) {
			action_listener.actionPerformed(
					new ActionEvent( this,
					ActionEvent.ACTION_PERFORMED, null ));
		}
	}
	public void mousePressed( MouseEvent e ) {
		mouse_pressed = true; repaint();
	}
	public void mouseReleased( MouseEvent e ) {
		mouse_pressed = false; repaint();
	}
	public void mouseEntered( MouseEvent e ) {
		mouse_entered = true; repaint();
	}
	public void mouseExited( MouseEvent e ) {
		mouse_entered = false; repaint();
	}

	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		ImageButton xpmb = new ImageButton(
			Xpm.get_image_producer( "Stop22x22" ), "Pause" );
		Frame frame = new Frame();
		frame.setLayout( new BorderLayout() );
		frame.add( "Center", xpmb );
		frame.pack(); frame.setVisible( true );
		while( true ) {
			try { Thread.sleep( 5000 ); }
			catch ( Exception e ) {}
			xpmb.setEnabled( ! xpmb.isEnabled() );
		}
	}
}
